"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateToken = exports.createToken = void 0;
const database_1 = require("../database");
const service_1 = require("../principal/service");
const http_errors_1 = require("@curveball/http-errors");
const crypto_1 = require("../crypto");
/**
 * 2 hour token timeout
 */
const tokenTTL = 7200;
/**
 * This function will create a unique token then store it in the database
 */
async function createToken(user, expiresIn) {
    const token = await (0, crypto_1.generateSecretToken)();
    const expiresAt = Math.floor(Date.now() / 1000) + (expiresIn ?? tokenTTL);
    await (0, database_1.default)('reset_password_token').insert({
        user_id: user.id,
        token,
        expires_at: expiresAt,
        created_at: Math.floor(Date.now() / 1000)
    });
    return {
        token,
        expires: new Date(expiresAt * 1000),
        ttl: tokenTTL,
    };
}
exports.createToken = createToken;
/**
 * Checks if a 'password reset token' is valid, and returns the associated user.
 * This function only works once for every token.
 * After calling this function, the token automatically gets deleted.
 */
async function validateToken(token, dontExpire = false) {
    const result = await (0, database_1.default)('reset_password_token')
        .select()
        .where({ token })
        .andWhere('expires_at', '>', Math.floor(Date.now() / 1000))
        .first();
    if (!result) {
        throw new http_errors_1.BadRequest('Failed to validate token');
    }
    else {
        if (!dontExpire) {
            await (0, database_1.default)('reset_password_token').delete().where({ token });
        }
        const principalService = new service_1.PrincipalService('insecure');
        return principalService.findById(result.user_id, 'user');
    }
}
exports.validateToken = validateToken;
//# sourceMappingURL=service.js.map