import { LazyPrivilegeBox } from '../privilege/service';
import { App, BasePrincipal, Group, NewPrincipal, Principal, PrincipalStats, PrincipalType, User } from '../types';
/**
 * This class provides a wrapper around the principal service APIs.
 *
 * It ensures that the caller (the personal or app that's logged in) has
 * the appropriate privileges to read the associated data.
 */
export declare class PrincipalService {
    private privileges;
    /**
     * Set up the principal service.
     *
     * You must pass the privileges for the current user, so we can check if
     * the user is allowed to perform the requests.
     *
     * If the string 'insecure' is passed, no privileges will be checked.
     */
    constructor(privileges: LazyPrivilegeBox | 'insecure');
    findAll(type: 'user'): Promise<User[]>;
    findAll(type: 'group'): Promise<Group[]>;
    findAll(type: 'app'): Promise<App[]>;
    findAll(): Promise<Principal[]>;
    findByIdentity(identity: string): Promise<Principal>;
    save<T extends PrincipalType>(principal: BasePrincipal<T> | NewPrincipal<T>): Promise<BasePrincipal<T>>;
    findByExternalId(externalId: string, type: 'user'): Promise<User>;
    findByExternalId(externalId: string, type: 'group'): Promise<Group>;
    findByExternalId(externalId: string, type: 'app'): Promise<App>;
    findByExternalId(externalId: string): Promise<Principal>;
    /**
     * Finds a principal by its href'.
     *
     * This can be a string like /user/1, or a full url.
     * It can also be the uri listed in the 'identity' field.
     */
    findByHref(href: string): Promise<Principal>;
    /**
     * Find multiple principals.
     *
     * This function returns the principals as a map, index by their id.
     * This is a helper function typically used by other services to find large numbers
     * of joined principals from other tables fast.
     *
     * If any of the ids in the list is duplicated, they are de-duplicated here.
     * if any of the provided ids are not found, this function will error.
     */
    findMany(ids: number[]): Promise<Map<number, Principal>>;
    findById(id: number, type: 'user'): Promise<User>;
    findById(id: number, type: 'group'): Promise<Group>;
    findById(id: number, type: 'app'): Promise<App>;
    findById(id: number): Promise<Principal>;
    /**
     * Returns the list of members of a group
     */
    findMembers(group: Group): Promise<Principal[]>;
    addMember(group: Group, user: Principal): Promise<void>;
    replaceMembers(group: Group, users: Principal[]): Promise<void>;
    removeMember(group: Group, user: Principal): Promise<void>;
    /**
     * Returns a list of groups for which the principal is a member
     */
    findGroupsForPrincipal(principal: Principal): Promise<Group[]>;
}
/**
 * Returns true if more than 1 principal exists in the system.
 *
 * If there are 0 principals, this puts a12nserver in setup mode, which allows a
 * person to create the first user in the system, automatically activate it
 * and make them an admin.
 */
export declare function hasUsers(): Promise<boolean>;
export declare function isIdentityValid(identity: string): boolean;
export declare function getPrincipalStats(): Promise<PrincipalStats>;
/**
 * @private Only exported for testing purposes.
 */
export declare function getPathName(href: string): string;
