"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addPrivilegeForUser = exports.replacePrivilegeForUser = exports.findPrivilegeType = exports.findPrivilegeTypes = exports.LazyPrivilegeBox = exports.getImmediatePrivilegesForPrincipal = exports.findPrivilegesForResource = exports.get = void 0;
const database_1 = require("../database");
const service_1 = require("../principal/service");
const http_errors_1 = require("@curveball/http-errors");
/**
 * Returns all privileges associated with a single principal.
 */
async function get(who) {
    const box = new LazyPrivilegeBox(who);
    await box.ready();
    return box;
}
exports.get = get;
/**
 * Returns all privileges defined on a single resource.
 */
async function findPrivilegesForResource(resource) {
    const principalService = new service_1.PrincipalService('insecure');
    const records = await (0, database_1.default)('user_privileges')
        .select('*')
        .where({ resource }).orWhere({ resource: '*' });
    const principals = await principalService.findMany(records.map(record => record.user_id));
    return records.map(record => ({
        privilege: record.privilege,
        resource: record.resource,
        principal: principals.get(record.user_id),
    }));
}
exports.findPrivilegesForResource = findPrivilegesForResource;
/**
 * Get all the privileges assigned to a principal, excluding privileges
 * the principal inherited by being part of a group.
 */
async function getImmediatePrivilegesForPrincipal(principal) {
    const result = await (0, database_1.query)('SELECT resource, privilege FROM user_privileges WHERE user_id = ?', [principal.id]);
    return result.reduce((currentPrivileges, row) => {
        const privileges = Object.assign({}, currentPrivileges);
        // eslint-disable-next-line no-prototype-builtins
        if (privileges.hasOwnProperty(row.resource)) {
            if (privileges[row.resource].indexOf(row.privilege) === -1) {
                privileges[row.resource].push(row.privilege);
            }
        }
        else {
            privileges[row.resource] = [row.privilege];
        }
        return privileges;
    }, {});
}
exports.getImmediatePrivilegesForPrincipal = getImmediatePrivilegesForPrincipal;
/**
 * Helper class for checking privileges.
 *
 * This class represents the set of privileges for a user.
 * It mostly operates synchronously, but it loads privileges asynchronously.
 *
 * To ensure it's fully loaded await the ready() function.
 */
class LazyPrivilegeBox {
    constructor(who) {
        this.privileges = null;
        if (isContext(who)) {
            if (!who.auth.isLoggedIn()) {
                this.who = 'public';
            }
            else {
                this.who = who.auth.principal;
            }
        }
        else {
            this.who = who;
        }
        this.init = this.reload();
    }
    getAll() {
        if (this.privileges === null) {
            throw new Error('List of privileges have not been loaded');
        }
        return this.privileges;
    }
    has(privilege, resource = '*') {
        if (this.who === 'public')
            return false;
        if (this.who === 'insecure')
            return true;
        const privileges = this.getAll();
        return privileges['*']?.includes(privilege) || privileges[resource]?.includes(privilege) || privileges['*']?.includes('admin') || privileges[resource]?.includes('admin') || false;
    }
    require(privilege, resource = '*') {
        if (!this.has(privilege, resource)) {
            if (this.who === 'public') {
                throw new http_errors_1.Forbidden('You must be authenticated for this operation');
            }
            throw new http_errors_1.Forbidden(`The currently logged in user must have the "${privilege}" privilege on the "${resource}" to do this`);
        }
    }
    /**
     * Await the result of this function to ensure that privleges have been loaded.
     */
    async ready() {
        await this.init;
    }
    /**
     * Reloads privileges from the database
     */
    async reload() {
        if (this.who === 'insecure' || this.who === 'public') {
            this.privileges = {};
        }
        else {
            this.privileges = await getPrivilegesForPrincipal(this.who);
        }
        return this.privileges;
    }
    /**
     * Returns true if the passed principal matches principal associated with
     * the current set of privileges.
     */
    isPrincipal(principal) {
        if (this.who === 'insecure' || this.who === 'public')
            return false;
        return this.who.id === principal.id;
    }
}
exports.LazyPrivilegeBox = LazyPrivilegeBox;
function isContext(input) {
    return input.request !== undefined && input.response !== undefined;
}
/**
 * Returns the list of 'privilege types'
 */
async function findPrivilegeTypes() {
    const result = await (0, database_1.query)(`
  SELECT privileges.privilege, privileges.description FROM privileges
  UNION ALL
  SELECT DISTINCT user_privileges.privilege, null FROM user_privileges
  LEFT JOIN privileges
  ON privileges.privilege = user_privileges.privilege
  WHERE privileges.privilege IS NULL`);
    return result;
}
exports.findPrivilegeTypes = findPrivilegeTypes;
async function findPrivilegeType(privilege) {
    const result = await (0, database_1.default)('privileges')
        .select('*')
        .where({ privilege });
    if (result.length !== 1) {
        return {
            privilege: privilege,
            description: ''
        };
    }
    return result[0];
}
exports.findPrivilegeType = findPrivilegeType;
async function replacePrivilegeForUser(user, privilegeMap) {
    await database_1.default.transaction(async (trx) => {
        await trx.raw('DELETE FROM user_privileges WHERE user_id = ?', [user.id]);
        for (const [resource, privileges] of Object.entries(privilegeMap)) {
            for (const privilege of privileges) {
                await trx('user_privileges').insert({
                    user_id: user.id,
                    privilege,
                    resource,
                });
            }
        }
        await trx.commit();
    });
}
exports.replacePrivilegeForUser = replacePrivilegeForUser;
async function addPrivilegeForUser(user, privilege, resource) {
    await (0, database_1.default)('user_privileges').insert({
        user_id: user.id,
        privilege,
        resource,
    });
}
exports.addPrivilegeForUser = addPrivilegeForUser;
/**
 * Returns a list of all group principals for a user.
 *
 * This basically returns a list of ids of all groups that a user is a part of,
 * recursively, and also includes the id of the user itself.
 */
async function getRecursiveGroupIds(principalId) {
    const q = 'SELECT group_id FROM group_members WHERE user_id = ?';
    const result = (await (0, database_1.query)(q, [principalId]));
    const ids = [
        principalId
    ];
    for (const row of result) {
        ids.push(...await getRecursiveGroupIds(row.group_id));
    }
    return ids;
}
async function getPrivilegesForPrincipal(principal) {
    const recursiveGroupIds = await getRecursiveGroupIds(principal.id);
    if (principal.type === 'user')
        recursiveGroupIds.push(await getAllUsersGroupId());
    const result = await (0, database_1.query)(`SELECT resource, privilege FROM user_privileges WHERE user_id IN (${recursiveGroupIds.map(_ => '?').join(',')})`, [...recursiveGroupIds]);
    return result.reduce((currentPrivileges, row) => {
        const privileges = Object.assign({}, currentPrivileges);
        // eslint-disable-next-line no-prototype-builtins
        if (privileges.hasOwnProperty(row.resource)) {
            if (privileges[row.resource].indexOf(row.privilege) === -1) {
                privileges[row.resource].push(row.privilege);
            }
        }
        else {
            privileges[row.resource] = [row.privilege];
        }
        return privileges;
    }, {});
}
let allUsersGroupId = null;
/**
 * Returns the set of privileges for the $all group
 */
async function getAllUsersGroupId() {
    if (allUsersGroupId)
        return allUsersGroupId;
    const allPrincipal = await (0, database_1.default)('principals')
        .select('id')
        .first()
        .where({
        external_id: '$all',
        type: 3
    });
    if (!allPrincipal) {
        throw new Error('Could not find the $all group in the database!');
    }
    allUsersGroupId = allPrincipal.id;
    return allUsersGroupId;
}
//# sourceMappingURL=service.js.map