"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.updateLastUse = exports.findByUserAndApp = exports.findByUser = exports.setPermissions = void 0;
const database_1 = require("../database");
const service_1 = require("../principal/service");
const http_errors_1 = require("@curveball/http-errors");
/**
 * This function will set the permissions an app has on behalf of a user.
 *
 * If permissions already existed, the new list of permissions will be merged
 * with the old ones.
 */
async function setPermissions(app, user, scope) {
    await database_1.default.transaction(async (trx) => {
        const userAppPermission = await trx('user_app_permissions')
            .first()
            .where({
            user_id: user.id,
            app_id: app.id
        })
            .forUpdate();
        if (!userAppPermission) {
            await trx('user_app_permissions').insert({
                app_id: app.id,
                user_id: user.id,
                scope: scope.length === 0 ? null : scope.join(' '),
                created_at: Date.now(),
                modified_at: Date.now(),
            });
        }
        else {
            const newScopes = new Set([
                ...scope,
                ...(userAppPermission.scope ? userAppPermission.scope.split(' ') : []),
            ]);
            const newScopeStr = Array.from(newScopes).sort().join(' ');
            if (newScopeStr !== userAppPermission.scope) {
                await trx('user_app_permissions').update({
                    scope: newScopes.size === 0 ? null : Array.from(newScopes).join(' '),
                    modified_at: Date.now(),
                });
            }
        }
    });
}
exports.setPermissions = setPermissions;
/**
 * Returns all the apps the given user has granted permissions to
 */
async function findByUser(user) {
    const principalService = new service_1.PrincipalService('insecure');
    const records = await (0, database_1.default)('user_app_permissions')
        .select()
        .where({ user_id: user.id });
    const apps = await principalService.findMany(records.map(record => record.app_id));
    return records.map(record => recordToModel(record, user, apps.get(record.app_id)));
}
exports.findByUser = findByUser;
/**
 * Returns all the apps the given user has granted permissions to
 */
async function findByUserAndApp(user, app) {
    const record = await (0, database_1.default)('user_app_permissions')
        .first()
        .where({
        user_id: user.id,
        app_id: app.id,
    });
    if (!record)
        throw new http_errors_1.NotFound(`User has not granted permission to app: ${app.href}`);
    return recordToModel(record, user, app);
}
exports.findByUserAndApp = findByUserAndApp;
/**
 * When an app generates an access token for a user, we update the lastUsedAt property.
 */
async function updateLastUse(user, app) {
    await (0, database_1.default)('user_app_permissions')
        .update({ last_used_at: Date.now() })
        .where({
        user_id: user.id,
        app_id: app.id
    });
}
exports.updateLastUse = updateLastUse;
function recordToModel(record, user, app) {
    return {
        app,
        user,
        scope: record.scope ? record.scope.split(' ') : [],
        createdAt: new Date(record.created_at),
        modifiedAt: new Date(record.modified_at),
        lastUsedAt: record.last_used_at === null ? null : new Date(record.last_used_at),
    };
}
//# sourceMappingURL=service.js.map